@@ -1,4 +1,4 @@
-(* $Id: print.ml 662 2004-05-25 20:57:28Z gerd $
+(* $Id: print.ml 745 2009-02-02 21:23:48Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -6,7 +6,7 @@
 (* Read a record-list structure and print it *)
 open Pxp_types;;
 open Pxp_document;;
-open Pxp_yacc;;
+open Pxp_tree_parser;;
 
 let print tree =
   iter_tree
@@ -33,7 +33,8 @@ let print tree =
 
 let main() =
   try
-    let dtd = parse_dtd_entity default_config (from_file "record.dtd") in
+    let dtd = 
+      Pxp_dtd_parser.parse_dtd_entity default_config (from_file "record.dtd") in
     let tree = 
       parse_content_entity default_config (from_channel stdin) dtd default_spec in
     print tree
