@@ -1,11 +1,11 @@
-(* $Id: validate.ml 712 2005-05-04 13:09:58Z gerd $
+(* $Id: validate.ml 745 2009-02-02 21:23:48Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
 
 
 open Pxp_document;;
-open Pxp_yacc;;
+open Pxp_tree_parser;;
 open Pxp_types;;
 
 let error_happened = ref false;;
