@@ -1,4 +1,4 @@
-(* $Id: pxp_aux.ml 690 2004-08-08 20:22:54Z gerd $
+(* $Id: pxp_aux.ml 738 2009-02-02 03:27:35Z gerd $
  * ----------------------------------------------------------------------
  * PXP: The polymorphic XML parser for Objective Caml.
  * Copyright by Gerd Stolpmann. See LICENSE for details.
@@ -9,7 +9,7 @@
 (* Lexing *)
 
 
-open Pxp_core_types
+open Pxp_core_types.I
 open Pxp_lexer_types
 open Pxp_lexers
 open Pxp_lib
