@@ -1,4 +1,4 @@
-(* $Id: pxp_ev_parser.ml 707 2004-09-04 17:25:35Z gerd $
+(* $Id: pxp_ev_parser.ml 741 2009-02-02 17:11:36Z gerd $
  * ----------------------------------------------------------------------
  * PXP: The polymorphic XML parser for Objective Caml.
  * Copyright by Gerd Stolpmann. See LICENSE for details.
@@ -15,10 +15,9 @@ open Pxp_aux
 
 (* The subclass event_parser for the event-based interface: *)
 
-class any_entity_id = object end ;;
-
 class event_parser init_dtd init_config init_event_handler 
                    init_want_start_doc init_pull_counter lit_root =
+  let null_entity_id = Pxp_dtd.Entity.create_entity_id() in
 object (self)
   inherit core_parser init_dtd init_config init_pull_counter
 
@@ -31,7 +30,7 @@ object (self)
   val mutable xml_standalone = false
 
   val mutable ep_root_element_seen = false
-  val mutable ep_elstack = stack_create (None,"","",new any_entity_id)
+  val mutable ep_elstack = stack_create (None,"","",null_entity_id)
   val mutable ep_early_events = []
 
 
@@ -44,7 +43,7 @@ object (self)
       raise(WF_error("Root element is not closed"));
 
 
-  method private init_for_xml_body() =
+  method private init_for_xml_body _ =
     if not init_done then begin
       if config.recognize_standalone_declaration then
 	dtd # set_standalone_declaration xml_standalone;
@@ -84,7 +83,7 @@ object (self)
        | None -> ()
     );
     if n_tags_open = 0 then begin
-      if ep_root_element_seen then
+      if ep_root_element_seen && not permit_any_content then
 	raise(WF_error("Document must consist of only one toplevel element"));
       ep_root_element_seen <- true;
       lit_root := name
@@ -174,7 +173,7 @@ object (self)
 	ep_early_events <- ep_early_events @ ev_list
     end
     else begin
-      let ev = E_pinstr_member(target,value,ent_id) in
+      let ev = E_pinstr(target,value,ent_id) in
       if init_done then
 	event_handler ev
       else
@@ -242,13 +241,13 @@ let process_entity
       (-1)
       lit_root
   in
-  let resolver = mgr # current_resolver in
   let init_lexer =
     match entry with
 	`Entry_document _     -> Document
       | `Entry_declarations _ -> failwith "Pxp_yacc.process_entity: bad entry point"
-      | `Entry_content _      -> Content
-      | `Entry_expr _         -> Content
+      | `Entry_content _         -> Content
+      | `Entry_element_content _ -> Content
+      | `Entry_expr _            -> Content
   in
   let en = mgr # current_entity in
   let gen_att_events = Some(cfg.escape_attributes <> None) in
@@ -288,7 +287,6 @@ let process_expr
       cfg mgr eh =
   let lit_root = ref "" in
   let pobj = new event_parser mgr#dtd cfg eh false (-1) lit_root in
-  let resolver = mgr # current_resolver in
   let en = mgr # current_entity in
   begin try
     let context = make_context ?first_token mgr in
@@ -317,8 +315,13 @@ let process_expr
 ;;
 
 
-let create_pull_parser
-      cfg entry mgr =
+let close_entities mgr =
+  let top = mgr#top_entity in
+  mgr # pop_entity_until top;
+  if top # is_open then ignore(top # close_entity)
+
+
+let create_pull_parser cfg entry mgr =
 
   (* Do control inversion with a queue serving as buffer, and a very special
    * kind of continuations 
@@ -343,12 +346,12 @@ let create_pull_parser
       100                   (* the number of loops until Interrupt_parsing *)
       lit_root
   in
-  let resolver = mgr # current_resolver in
   let init_lexer =
     match entry with
 	`Entry_document _     -> Document
       | `Entry_declarations _ -> failwith "Pxp_yacc.process_entity: bad entry point"
       | `Entry_content _      -> Content
+      | `Entry_element_content _ -> Content
       | `Entry_expr _         -> Content
   in
   let en = mgr # current_entity in
