@@ -1,4 +1,4 @@
-(* $Id: pxp_lexer_types.ml,v 1.12 2003/06/15 12:23:21 gerd Exp $
+(* $Id: pxp_lexer_types.ml 744 2009-02-02 21:10:02Z gerd $
  * ----------------------------------------------------------------------
  * PXP: The polymorphic XML parser for Objective Caml.
  * Copyright by Gerd Stolpmann. See LICENSE for details.
@@ -41,10 +41,7 @@ type prolog_token =
   | Pro_eof
 
 
-type entity_id = < >
-  (* The class without properties; but you can still compare if two objects
-   * are the same.
-   *)
+type entity_id = < pxp_magic_coercion : unit -> unit >
 
 type token = 
   | Begin_entity             (* Beginning of entity *)
@@ -90,7 +87,7 @@ type token =
   | Tag_beg of (string*entity_id)     (* <name *)
   | Tag_end of (string*entity_id)     (* </name *)
 
-  | PI        of (string*string)      (* <?name ... ?> *)
+  | PI        of (string*string*entity_id)      (* <?name ... ?> *)
   | PI_xml    of (prolog_token list)  (* <?xml ...?> *)
   | Cdata     of string               (* <![CDATA[...]]> *)
   | CRef      of int                  (* &#digits; *)
@@ -181,147 +178,55 @@ let string_of_tok tok =
   | DQuote -> "DQuote"
   | ERef_att _ -> "ERef_att"
 
+type lexbuf =
+    [ `Ocamllex of Lexing.lexbuf
+    | `Netulex of Netulex.Ulexing.lexbuf
+    ]
+
+class type lexer_factory =
+object
+  method encoding : Pxp_core_types.I.rep_encoding
+  method open_source : Pxp_reader.lexer_source -> lexer_obj
+  method open_string : string -> lexer_obj
+  method open_string_inplace : string -> lexer_obj
+end
+
+and lexer_obj =
+object
+  method factory : lexer_factory
+  method encoding : Pxp_core_types.I.rep_encoding
+  method open_source : Pxp_reader.lexer_source -> unit
+  method open_string : string -> unit
+  method open_string_inplace : string -> unit
+
+  method scan_document        : unit -> (token * lexers)
+  method scan_content         : unit -> (token * lexers)
+  method scan_within_tag      : unit -> (token * lexers)
+  method scan_document_type   : unit -> (token * lexers)
+  method scan_declaration     : unit -> (token * lexers)
+  method scan_comment         : unit -> lexers -> (token * lexers)
+  method scan_ignored_section : unit -> (token * lexers)
+  method detect_xml_pi        : unit -> bool
+  method scan_xml_pi          : unit -> prolog_token
+  method scan_pi_string       : unit -> string option
+  method scan_dtd_string      : unit -> token
+  method scan_content_string  : unit -> token
+  method scan_name_string     : unit -> token
+  method scan_for_crlf        : unit -> token
+  method scan_characters      : unit -> unit
+  method scan_character       : unit -> unit
+  method scan_tag_eb          : unit -> (token * lexers)
+  method scan_tag_eb_att      : unit -> bool -> (token * lexers)
+
+  method lexeme_length : int
+  method lexeme_char : int -> int
+  method lexeme : string
+  method lexeme_strlen : int
+  method sub_lexeme : int -> int -> string
+  method lexbuf : lexbuf
+end
+
 
 type lexer_set =
-    { lex_encoding         : Pxp_core_types.rep_encoding;
-      scan_document        : Lexing.lexbuf -> (token * lexers);
-      scan_content         : Lexing.lexbuf -> (token * lexers);
-      scan_within_tag      : Lexing.lexbuf -> (token * lexers);
-      scan_document_type   : Lexing.lexbuf -> (token * lexers);
-      scan_declaration     : Lexing.lexbuf -> (token * lexers);
-      scan_comment         : Lexing.lexbuf -> lexers -> (token * lexers);
-      scan_ignored_section : Lexing.lexbuf -> (token * lexers);
-      detect_xml_pi        : Lexing.lexbuf -> bool;
-      scan_xml_pi          : Lexing.lexbuf -> prolog_token;
-      scan_pi_string       : Lexing.lexbuf -> string option;
-      scan_dtd_string      : Lexing.lexbuf -> token;
-      scan_content_string  : Lexing.lexbuf -> token;
-      scan_name_string     : Lexing.lexbuf -> token;
-      scan_for_crlf        : Lexing.lexbuf -> token;
-      scan_characters      : Lexing.lexbuf -> unit;
-      scan_character       : Lexing.lexbuf -> unit;
-      scan_tag_eb          : Lexing.lexbuf -> (token * lexers);
-      scan_tag_eb_att      : Lexing.lexbuf -> bool -> (token * lexers);
-    }
-
-(* ======================================================================
- * History:
- * 
- * $Log: pxp_lexer_types.ml,v $
- * Revision 1.12  2003/06/15 12:23:21  gerd
- * 	Moving core type definitions to Pxp_core_types
- *
- * Revision 1.11  2002/08/31 23:26:20  gerd
- * 	Instead of three comment lexers for content, decl and document,
- * there is now only one unified comment lexer.
- * 	New scanners detect_xml_pi, scan_pi_string, scan_character.
- * 	Removed the scanner scan_only_xml_decl.
- *
- * Revision 1.10  2002/08/28 23:54:34  gerd
- * 	Support for new lexer definition style.
- *
- * Revision 1.9  2002/08/05 22:33:33  gerd
- * 	New token LineEnd_att for newline characters inside
- * attribute values (event-based parser).
- *
- * Revision 1.8  2002/08/03 17:54:47  gerd
- * 	Support for event-based parsing of attribute values: New
- * lexers Tag_eb, Tag_eb_att, and the new pseudo lexer Within_tag_entry
- * (which immediately changes the lexer to either Within_tag or Tag_eb).
- * New tokens SQuote, DQuote, ERef_att.
- *
- * Revision 1.7  2002/07/14 23:04:17  gerd
- * 	New lexer "Closed", used after an entity is closed.
- * 	New tokens Lcurly, LLcurly, Rcurly, RRcurly.
- *
- * Revision 1.6  2002/03/13 22:45:42  gerd
- * 	Improved Pxp_lexing.
- *
- * Revision 1.5  2001/06/28 22:42:07  gerd
- * 	Fixed minor problems:
- * 	- Comments must be contained in one entity
- * 	- Pxp_document.document is now initialized with encoding.
- *           the DTD encoding may be initialized too late.
- *
- * Revision 1.4  2000/10/01 19:47:53  gerd
- * 	New functions: sub_lexeme, fast_lexing_from_string,
- * reuse_lexing_from_string.
- *
- * Revision 1.3  2000/09/21 21:28:16  gerd
- * 	New token IgnoreLineEnd: simplifies line counting, and
- * corrects a bug.
- *
- * Revision 1.2  2000/08/18 20:14:31  gerd
- * 	Comment -> Comment_begin, Comment_material, Comment_end.
- *
- * Revision 1.1  2000/05/29 23:48:38  gerd
- * 	Changed module names:
- * 		Markup_aux          into Pxp_aux
- * 		Markup_codewriter   into Pxp_codewriter
- * 		Markup_document     into Pxp_document
- * 		Markup_dtd          into Pxp_dtd
- * 		Markup_entity       into Pxp_entity
- * 		Markup_lexer_types  into Pxp_lexer_types
- * 		Markup_reader       into Pxp_reader
- * 		Markup_types        into Pxp_types
- * 		Markup_yacc         into Pxp_yacc
- * See directory "compatibility" for (almost) compatible wrappers emulating
- * Markup_document, Markup_dtd, Markup_reader, Markup_types, and Markup_yacc.
- *
- * ======================================================================
- * Old logs from markup_lexer_types.ml:
- *
- * Revision 1.6  2000/05/29 21:14:57  gerd
- * 	Changed the type 'encoding' into a polymorphic variant.
- *
- * Revision 1.5  2000/05/20 20:31:40  gerd
- * 	Big change: Added support for various encodings of the
- * internal representation.
- *
- * Revision 1.4  2000/05/14 17:45:36  gerd
- * 	Bugfix.
- *
- * Revision 1.3  2000/05/14 17:35:12  gerd
- * 	Conditional_begin, _end, and _body have an entity_id.
- *
- * Revision 1.2  2000/05/08 21:59:06  gerd
- * 	New token Bof (beginning of file).
- *
- * Revision 1.1  2000/05/06 23:21:49  gerd
- * 	Initial revision.
- *
- *
- * ======================================================================
- *
- * DERIVED FROM REVISION 1.4 of markup_lexer_types_shadow.ml
- *
- * Revision 1.4  2000/04/30 18:19:04  gerd
- * 	Added new tokens.
- *
- * Revision 1.3  1999/08/31 19:13:31  gerd
- * 	Added checks on proper PE nesting. The idea is that tokens such
- * as Decl_element and Decl_rangle carry an entity ID with them. This ID
- * is simply an object of type < >, i.e. you can only test on identity.
- * The lexer always produces tokens with a dummy ID because it does not
- * know which entity is the current one. The entity layer replaces the dummy
- * ID with the actual ID. The parser checks that the IDs of pairs such as
- * Decl_element and Decl_rangle are the same; otherwise a Validation_error
- * is produced.
- *
- * Revision 1.2  1999/08/10 21:35:08  gerd
- * 	The XML/encoding declaration at the beginning of entities is
- * evaluated. In particular, entities have now a method "xml_declaration"
- * which returns the name/value pairs of such a declaration. The "encoding"
- * setting is interpreted by the entity itself; "version", and "standalone"
- * are interpreted by Markup_yacc.parse_document_entity. Other settings
- * are ignored (this does not conform to the standard; the standard prescribes
- * that "version" MUST be given in the declaration of document; "standalone"
- * and "encoding" CAN be declared; no other settings are allowed).
- * 	TODO: The user should be warned if the standard is not exactly
- * fulfilled. -- The "standalone" property is not checked yet.
- *
- * Revision 1.1  1999/08/10 00:35:51  gerd
- * 	Initial revision.
- *
- * 
- *)
+    { scan_name_string : Lexing.lexbuf -> token }
+  (* DEPRECATED. Only exists because WDialog needs it. *)
